"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from moovio_sdk import utils
from moovio_sdk._hooks import HookContext
from moovio_sdk.models import components, errors, operations
from moovio_sdk.types import BaseModel, OptionalNullable, UNSET
from moovio_sdk.utils import get_security_from_env
from moovio_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union, cast


class Onboarding(BaseSDK):
    def create_invite(
        self,
        *,
        scopes: List[components.ApplicationScope],
        capabilities: List[components.CapabilityID],
        fee_plan_codes: List[str],
        return_url: Optional[str] = None,
        terms_of_service_url: Optional[str] = None,
        grant_scopes: Optional[List[components.ApplicationScope]] = None,
        prefill: Optional[
            Union[components.CreateAccount, components.CreateAccountTypedDict]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateOnboardingInviteResponse:
        r"""Create an invitation containing a unique link that allows the recipient to onboard their organization with Moov.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts.write` scope.

        :param scopes: List of [scopes](https://docs.moov.io/api/authentication/scopes/) you request to use on this
            account. These values are used to determine what can be done with the account onboarded.
        :param capabilities: List of [capabilities](https://docs.moov.io/guides/accounts/capabilities/) you intend to request for this
            account. These values are used to determine what information to collect from the user during onboarding.
        :param fee_plan_codes: List of fee plan codes to assign the account created by the invitee.
        :param return_url: Optional URL to redirect the user to after they complete the onboarding process.
        :param terms_of_service_url: Optional URL to your organization's terms of service.
        :param grant_scopes: List of [scopes](https://docs.moov.io/api/authentication/scopes/) you grant to allow being used
            by the new account on yourself. These values are used to determine what the account onboarded can do.
        :param prefill:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = components.OnboardingInviteRequest(
            return_url=return_url,
            terms_of_service_url=terms_of_service_url,
            scopes=scopes,
            grant_scopes=grant_scopes,
            capabilities=capabilities,
            fee_plan_codes=fee_plan_codes,
            prefill=utils.get_pydantic_model(
                prefill, Optional[components.CreateAccount]
            ),
        )

        req = self._build_request(
            method="POST",
            path="/onboarding-invites",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.CreateOnboardingInviteGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.OnboardingInviteRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createOnboardingInvite",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateOnboardingInviteResponse(
                result=unmarshal_json_response(components.OnboardingInvite, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.OnboardingInviteErrorData, http_res
            )
            raise errors.OnboardingInviteError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def create_invite_async(
        self,
        *,
        scopes: List[components.ApplicationScope],
        capabilities: List[components.CapabilityID],
        fee_plan_codes: List[str],
        return_url: Optional[str] = None,
        terms_of_service_url: Optional[str] = None,
        grant_scopes: Optional[List[components.ApplicationScope]] = None,
        prefill: Optional[
            Union[components.CreateAccount, components.CreateAccountTypedDict]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateOnboardingInviteResponse:
        r"""Create an invitation containing a unique link that allows the recipient to onboard their organization with Moov.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts.write` scope.

        :param scopes: List of [scopes](https://docs.moov.io/api/authentication/scopes/) you request to use on this
            account. These values are used to determine what can be done with the account onboarded.
        :param capabilities: List of [capabilities](https://docs.moov.io/guides/accounts/capabilities/) you intend to request for this
            account. These values are used to determine what information to collect from the user during onboarding.
        :param fee_plan_codes: List of fee plan codes to assign the account created by the invitee.
        :param return_url: Optional URL to redirect the user to after they complete the onboarding process.
        :param terms_of_service_url: Optional URL to your organization's terms of service.
        :param grant_scopes: List of [scopes](https://docs.moov.io/api/authentication/scopes/) you grant to allow being used
            by the new account on yourself. These values are used to determine what the account onboarded can do.
        :param prefill:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = components.OnboardingInviteRequest(
            return_url=return_url,
            terms_of_service_url=terms_of_service_url,
            scopes=scopes,
            grant_scopes=grant_scopes,
            capabilities=capabilities,
            fee_plan_codes=fee_plan_codes,
            prefill=utils.get_pydantic_model(
                prefill, Optional[components.CreateAccount]
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/onboarding-invites",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.CreateOnboardingInviteGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.OnboardingInviteRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createOnboardingInvite",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateOnboardingInviteResponse(
                result=unmarshal_json_response(components.OnboardingInvite, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.OnboardingInviteErrorData, http_res
            )
            raise errors.OnboardingInviteError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def list_invites(
        self,
        *,
        request: Union[
            operations.ListOnboardingInvitesRequest,
            operations.ListOnboardingInvitesRequestTypedDict,
        ] = operations.ListOnboardingInvitesRequest(),
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListOnboardingInvitesResponse:
        r"""List all the onboarding invites created by the caller's account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts.read` scope.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.ListOnboardingInvitesRequest)
        request = cast(operations.ListOnboardingInvitesRequest, request)

        req = self._build_request(
            method="GET",
            path="/onboarding-invites",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListOnboardingInvitesGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listOnboardingInvites",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListOnboardingInvitesResponse(
                result=unmarshal_json_response(
                    List[components.OnboardingInvite], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_invites_async(
        self,
        *,
        request: Union[
            operations.ListOnboardingInvitesRequest,
            operations.ListOnboardingInvitesRequestTypedDict,
        ] = operations.ListOnboardingInvitesRequest(),
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListOnboardingInvitesResponse:
        r"""List all the onboarding invites created by the caller's account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts.read` scope.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.ListOnboardingInvitesRequest)
        request = cast(operations.ListOnboardingInvitesRequest, request)

        req = self._build_request_async(
            method="GET",
            path="/onboarding-invites",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListOnboardingInvitesGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listOnboardingInvites",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListOnboardingInvitesResponse(
                result=unmarshal_json_response(
                    List[components.OnboardingInvite], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get_invite(
        self,
        *,
        code: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetOnboardingInviteResponse:
        r"""Retrieve details about an onboarding invite.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts.read` scope.

        :param code:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetOnboardingInviteRequest(
            code=code,
        )

        req = self._build_request(
            method="GET",
            path="/onboarding-invites/{code}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetOnboardingInviteGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getOnboardingInvite",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetOnboardingInviteResponse(
                result=unmarshal_json_response(components.OnboardingInvite, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_invite_async(
        self,
        *,
        code: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetOnboardingInviteResponse:
        r"""Retrieve details about an onboarding invite.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts.read` scope.

        :param code:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetOnboardingInviteRequest(
            code=code,
        )

        req = self._build_request_async(
            method="GET",
            path="/onboarding-invites/{code}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetOnboardingInviteGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getOnboardingInvite",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetOnboardingInviteResponse(
                result=unmarshal_json_response(components.OnboardingInvite, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def revoke_invite(
        self,
        *,
        code: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.RevokeOnboardingInviteResponse:
        r"""Revoke an onboarding invite, rendering the invitation link unusable.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts.write` scope.

        :param code:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.RevokeOnboardingInviteRequest(
            code=code,
        )

        req = self._build_request(
            method="DELETE",
            path="/onboarding-invites/{code}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            _globals=operations.RevokeOnboardingInviteGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="revokeOnboardingInvite",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "204", "*"):
            return operations.RevokeOnboardingInviteResponse(
                headers=utils.get_response_headers(http_res.headers)
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def revoke_invite_async(
        self,
        *,
        code: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.RevokeOnboardingInviteResponse:
        r"""Revoke an onboarding invite, rendering the invitation link unusable.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts.write` scope.

        :param code:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.RevokeOnboardingInviteRequest(
            code=code,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/onboarding-invites/{code}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            _globals=operations.RevokeOnboardingInviteGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="revokeOnboardingInvite",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "204", "*"):
            return operations.RevokeOnboardingInviteResponse(
                headers=utils.get_response_headers(http_res.headers)
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import datetime
from moovio_sdk import utils
from moovio_sdk._hooks import HookContext
from moovio_sdk.models import components, errors, operations
from moovio_sdk.types import OptionalNullable, UNSET
from moovio_sdk.utils import get_security_from_env
from moovio_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional


class WalletTransactions(BaseSDK):
    def list(
        self,
        *,
        account_id: str,
        wallet_id: str,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        transaction_type: Optional[components.WalletTransactionType] = None,
        transaction_types: Optional[List[components.WalletTransactionType]] = None,
        source_type: Optional[components.WalletTransactionSourceType] = None,
        source_id: Optional[str] = None,
        status: Optional[components.WalletTransactionStatus] = None,
        created_start_date_time: Optional[datetime] = None,
        created_end_date_time: Optional[datetime] = None,
        completed_start_date_time: Optional[datetime] = None,
        completed_end_date_time: Optional[datetime] = None,
        sweep_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListWalletTransactionsResponse:
        r"""List all the transactions associated with a particular Moov wallet.

        Read our [wallet transactions guide](https://docs.moov.io/guides/sources/wallets/transactions/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/wallets.read` scope.

        :param account_id:
        :param wallet_id:
        :param skip:
        :param count:
        :param transaction_type: Optional parameter to filter by transaction type.
        :param transaction_types: Optional, comma-separated parameter to filter by transaction types.
        :param source_type: Optional parameter to filter by source type (i.e. transfer, dispute, issuing-transaction).
        :param source_id: Optional parameter to filter by source ID.
        :param status: Optional parameter to filter by status (`pending` or `completed`).
        :param created_start_date_time: Optional date-time which inclusively filters all transactions created after this date-time.
        :param created_end_date_time: Optional date-time which exclusively filters all transactions created before this date-time.
        :param completed_start_date_time: Optional date-time which inclusively filters all transactions completed after this date-time.
        :param completed_end_date_time: Optional date-time which exclusively filters all transactions completed before this date-time.
        :param sweep_id: Optional ID to filter for transactions accrued in a sweep.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListWalletTransactionsRequest(
            account_id=account_id,
            skip=skip,
            count=count,
            wallet_id=wallet_id,
            transaction_type=transaction_type,
            transaction_types=transaction_types,
            source_type=source_type,
            source_id=source_id,
            status=status,
            created_start_date_time=created_start_date_time,
            created_end_date_time=created_end_date_time,
            completed_start_date_time=completed_start_date_time,
            completed_end_date_time=completed_end_date_time,
            sweep_id=sweep_id,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/wallets/{walletID}/transactions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListWalletTransactionsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listWalletTransactions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "422", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListWalletTransactionsResponse(
                result=unmarshal_json_response(
                    List[components.WalletTransaction], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.ListWalletTransactionsValidationErrorData, http_res
            )
            raise errors.ListWalletTransactionsValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        account_id: str,
        wallet_id: str,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        transaction_type: Optional[components.WalletTransactionType] = None,
        transaction_types: Optional[List[components.WalletTransactionType]] = None,
        source_type: Optional[components.WalletTransactionSourceType] = None,
        source_id: Optional[str] = None,
        status: Optional[components.WalletTransactionStatus] = None,
        created_start_date_time: Optional[datetime] = None,
        created_end_date_time: Optional[datetime] = None,
        completed_start_date_time: Optional[datetime] = None,
        completed_end_date_time: Optional[datetime] = None,
        sweep_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListWalletTransactionsResponse:
        r"""List all the transactions associated with a particular Moov wallet.

        Read our [wallet transactions guide](https://docs.moov.io/guides/sources/wallets/transactions/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/wallets.read` scope.

        :param account_id:
        :param wallet_id:
        :param skip:
        :param count:
        :param transaction_type: Optional parameter to filter by transaction type.
        :param transaction_types: Optional, comma-separated parameter to filter by transaction types.
        :param source_type: Optional parameter to filter by source type (i.e. transfer, dispute, issuing-transaction).
        :param source_id: Optional parameter to filter by source ID.
        :param status: Optional parameter to filter by status (`pending` or `completed`).
        :param created_start_date_time: Optional date-time which inclusively filters all transactions created after this date-time.
        :param created_end_date_time: Optional date-time which exclusively filters all transactions created before this date-time.
        :param completed_start_date_time: Optional date-time which inclusively filters all transactions completed after this date-time.
        :param completed_end_date_time: Optional date-time which exclusively filters all transactions completed before this date-time.
        :param sweep_id: Optional ID to filter for transactions accrued in a sweep.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListWalletTransactionsRequest(
            account_id=account_id,
            skip=skip,
            count=count,
            wallet_id=wallet_id,
            transaction_type=transaction_type,
            transaction_types=transaction_types,
            source_type=source_type,
            source_id=source_id,
            status=status,
            created_start_date_time=created_start_date_time,
            created_end_date_time=created_end_date_time,
            completed_start_date_time=completed_start_date_time,
            completed_end_date_time=completed_end_date_time,
            sweep_id=sweep_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/wallets/{walletID}/transactions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListWalletTransactionsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listWalletTransactions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "422", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListWalletTransactionsResponse(
                result=unmarshal_json_response(
                    List[components.WalletTransaction], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.ListWalletTransactionsValidationErrorData, http_res
            )
            raise errors.ListWalletTransactionsValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        account_id: str,
        wallet_id: str,
        transaction_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetWalletTransactionResponse:
        r"""Get details on a specific wallet transaction.

        Read our [wallet transactions guide](https://docs.moov.io/guides/sources/wallets/transactions/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/wallets.read` scope.

        :param account_id:
        :param wallet_id:
        :param transaction_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetWalletTransactionRequest(
            account_id=account_id,
            wallet_id=wallet_id,
            transaction_id=transaction_id,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/wallets/{walletID}/transactions/{transactionID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetWalletTransactionGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getWalletTransaction",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetWalletTransactionResponse(
                result=unmarshal_json_response(components.WalletTransaction, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        account_id: str,
        wallet_id: str,
        transaction_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetWalletTransactionResponse:
        r"""Get details on a specific wallet transaction.

        Read our [wallet transactions guide](https://docs.moov.io/guides/sources/wallets/transactions/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/wallets.read` scope.

        :param account_id:
        :param wallet_id:
        :param transaction_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetWalletTransactionRequest(
            account_id=account_id,
            wallet_id=wallet_id,
            transaction_id=transaction_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/wallets/{walletID}/transactions/{transactionID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetWalletTransactionGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getWalletTransaction",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetWalletTransactionResponse(
                result=unmarshal_json_response(components.WalletTransaction, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .individualnameupdate import IndividualNameUpdate, IndividualNameUpdateTypedDict
from moovio_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PhoneTypedDict(TypedDict):
    number: NotRequired[str]
    country_code: NotRequired[str]


class Phone(BaseModel):
    number: Optional[str] = None

    country_code: Annotated[Optional[str], pydantic.Field(alias="countryCode")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["number", "countryCode"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class UpdateRepresentativeAddressTypedDict(TypedDict):
    address_line1: NotRequired[str]
    address_line2: NotRequired[str]
    city: NotRequired[str]
    state_or_province: NotRequired[str]
    postal_code: NotRequired[str]
    country: NotRequired[str]


class UpdateRepresentativeAddress(BaseModel):
    address_line1: Annotated[Optional[str], pydantic.Field(alias="addressLine1")] = None

    address_line2: Annotated[Optional[str], pydantic.Field(alias="addressLine2")] = None

    city: Optional[str] = None

    state_or_province: Annotated[
        Optional[str], pydantic.Field(alias="stateOrProvince")
    ] = None

    postal_code: Annotated[Optional[str], pydantic.Field(alias="postalCode")] = None

    country: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "addressLine1",
                "addressLine2",
                "city",
                "stateOrProvince",
                "postalCode",
                "country",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class UpdateRepresentativeBirthDateTypedDict(TypedDict):
    day: NotRequired[int]
    month: NotRequired[int]
    year: NotRequired[int]


class UpdateRepresentativeBirthDate(BaseModel):
    day: Optional[int] = None

    month: Optional[int] = None

    year: Optional[int] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["day", "month", "year"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class UpdateRepresentativeSsnTypedDict(TypedDict):
    full: NotRequired[str]
    last_four: NotRequired[str]


class UpdateRepresentativeSsn(BaseModel):
    full: Optional[str] = None

    last_four: Annotated[Optional[str], pydantic.Field(alias="lastFour")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["full", "lastFour"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class UpdateRepresentativeItinTypedDict(TypedDict):
    full: NotRequired[str]
    last_four: NotRequired[str]


class UpdateRepresentativeItin(BaseModel):
    full: Optional[str] = None

    last_four: Annotated[Optional[str], pydantic.Field(alias="lastFour")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["full", "lastFour"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class UpdateRepresentativeGovernmentIDTypedDict(TypedDict):
    ssn: NotRequired[UpdateRepresentativeSsnTypedDict]
    itin: NotRequired[UpdateRepresentativeItinTypedDict]


class UpdateRepresentativeGovernmentID(BaseModel):
    ssn: Optional[UpdateRepresentativeSsn] = None

    itin: Optional[UpdateRepresentativeItin] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["ssn", "itin"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ResponsibilitiesTypedDict(TypedDict):
    r"""Describes the job responsibilities of a business representative."""

    is_controller: NotRequired[bool]
    r"""Indicates whether this individual has significant management responsibilities within the business."""
    is_owner: NotRequired[bool]
    r"""If `true`, this field indicates that the individual has a business ownership stake of at least 25% in the
    business. If the representative does not own at least 25% of the business, this field should be `false`.
    """
    ownership_percentage: NotRequired[int]
    r"""The percentage of ownership this individual has in the business (required if `isOwner` is `true`)."""
    job_title: NotRequired[str]


class Responsibilities(BaseModel):
    r"""Describes the job responsibilities of a business representative."""

    is_controller: Annotated[Optional[bool], pydantic.Field(alias="isController")] = (
        None
    )
    r"""Indicates whether this individual has significant management responsibilities within the business."""

    is_owner: Annotated[Optional[bool], pydantic.Field(alias="isOwner")] = None
    r"""If `true`, this field indicates that the individual has a business ownership stake of at least 25% in the
    business. If the representative does not own at least 25% of the business, this field should be `false`.
    """

    ownership_percentage: Annotated[
        Optional[int], pydantic.Field(alias="ownershipPercentage")
    ] = None
    r"""The percentage of ownership this individual has in the business (required if `isOwner` is `true`)."""

    job_title: Annotated[Optional[str], pydantic.Field(alias="jobTitle")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["isController", "isOwner", "ownershipPercentage", "jobTitle"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class UpdateRepresentativeTypedDict(TypedDict):
    name: NotRequired[IndividualNameUpdateTypedDict]
    phone: NotRequired[Nullable[PhoneTypedDict]]
    email: NotRequired[Nullable[str]]
    address: NotRequired[Nullable[UpdateRepresentativeAddressTypedDict]]
    birth_date: NotRequired[Nullable[UpdateRepresentativeBirthDateTypedDict]]
    government_id: NotRequired[Nullable[UpdateRepresentativeGovernmentIDTypedDict]]
    responsibilities: NotRequired[Nullable[ResponsibilitiesTypedDict]]


class UpdateRepresentative(BaseModel):
    name: Optional[IndividualNameUpdate] = None

    phone: OptionalNullable[Phone] = UNSET

    email: OptionalNullable[str] = UNSET

    address: OptionalNullable[UpdateRepresentativeAddress] = UNSET

    birth_date: Annotated[
        OptionalNullable[UpdateRepresentativeBirthDate],
        pydantic.Field(alias="birthDate"),
    ] = UNSET

    government_id: Annotated[
        OptionalNullable[UpdateRepresentativeGovernmentID],
        pydantic.Field(alias="governmentID"),
    ] = UNSET

    responsibilities: OptionalNullable[Responsibilities] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "name",
                "phone",
                "email",
                "address",
                "birthDate",
                "governmentID",
                "responsibilities",
            ]
        )
        nullable_fields = set(
            [
                "phone",
                "email",
                "address",
                "birthDate",
                "governmentID",
                "responsibilities",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    Phone.model_rebuild()
except NameError:
    pass
try:
    UpdateRepresentativeAddress.model_rebuild()
except NameError:
    pass
try:
    UpdateRepresentativeSsn.model_rebuild()
except NameError:
    pass
try:
    UpdateRepresentativeItin.model_rebuild()
except NameError:
    pass
try:
    Responsibilities.model_rebuild()
except NameError:
    pass
try:
    UpdateRepresentative.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class PaymentMethodType(str, Enum):
    r"""The payment method type that represents a payment rail and directionality"""

    MOOV_WALLET = "moov-wallet"
    ACH_DEBIT_FUND = "ach-debit-fund"
    ACH_DEBIT_COLLECT = "ach-debit-collect"
    ACH_CREDIT_STANDARD = "ach-credit-standard"
    ACH_CREDIT_SAME_DAY = "ach-credit-same-day"
    RTP_CREDIT = "rtp-credit"
    CARD_PAYMENT = "card-payment"
    PUSH_TO_CARD = "push-to-card"
    PULL_FROM_CARD = "pull-from-card"
    APPLE_PAY = "apple-pay"
    CARD_PRESENT_PAYMENT = "card-present-payment"
    INSTANT_BANK_CREDIT = "instant-bank-credit"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from .createinvoicelineitems import (
    CreateInvoiceLineItems,
    CreateInvoiceLineItemsTypedDict,
)
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateInvoiceTypedDict(TypedDict):
    customer_account_id: str
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""
    line_items: CreateInvoiceLineItemsTypedDict
    r"""A collection of line items for an invoice."""
    description: NotRequired[str]
    invoice_date: NotRequired[datetime]
    due_date: NotRequired[datetime]
    tax_amount: NotRequired[AmountDecimalTypedDict]


class CreateInvoice(BaseModel):
    customer_account_id: Annotated[str, pydantic.Field(alias="customerAccountID")]
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""

    line_items: Annotated[CreateInvoiceLineItems, pydantic.Field(alias="lineItems")]
    r"""A collection of line items for an invoice."""

    description: Optional[str] = None

    invoice_date: Annotated[Optional[datetime], pydantic.Field(alias="invoiceDate")] = (
        None
    )

    due_date: Annotated[Optional[datetime], pydantic.Field(alias="dueDate")] = None

    tax_amount: Annotated[
        Optional[AmountDecimal], pydantic.Field(alias="taxAmount")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "invoiceDate", "dueDate", "taxAmount"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreateInvoice.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .applicationscope import ApplicationScope
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ShareScopesTypedDict(TypedDict):
    r"""Describes the scopes being shared from a subject account to a principal account."""

    principal_account_id: str
    r"""The account ID that will receive access to the scopes."""
    allow_scopes: NotRequired[List[ApplicationScope]]
    r"""The list of scopes to share with the principal account. If none are provided, all intersecting scopes are added."""


class ShareScopes(BaseModel):
    r"""Describes the scopes being shared from a subject account to a principal account."""

    principal_account_id: Annotated[str, pydantic.Field(alias="principalAccountID")]
    r"""The account ID that will receive access to the scopes."""

    allow_scopes: Annotated[
        Optional[List[ApplicationScope]], pydantic.Field(alias="allowScopes")
    ] = None
    r"""The list of scopes to share with the principal account. If none are provided, all intersecting scopes are added."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["allowScopes"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    ShareScopes.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class GuestProfileTypedDict(TypedDict):
    r"""Describes a guest account profile."""

    name: str
    r"""The name associated with the guest account.
    This will default to \"Guest {accountIDfirst8}\" if no other name is provided.
    """
    phone: NotRequired[PhoneNumberTypedDict]
    email: NotRequired[str]


class GuestProfile(BaseModel):
    r"""Describes a guest account profile."""

    name: str
    r"""The name associated with the guest account.
    This will default to \"Guest {accountIDfirst8}\" if no other name is provided.
    """

    phone: Optional[PhoneNumber] = None

    email: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["phone", "email"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WebhookDataNetworkIDUpdatedTypedDict(TypedDict):
    account_id: str
    visa_mid: NotRequired[str]
    mastercard_mid: NotRequired[str]
    discover_mid: NotRequired[str]
    amex_mid: NotRequired[str]
    updated_on: NotRequired[datetime]


class WebhookDataNetworkIDUpdated(BaseModel):
    account_id: Annotated[str, pydantic.Field(alias="accountID")]

    visa_mid: Annotated[Optional[str], pydantic.Field(alias="visaMid")] = None

    mastercard_mid: Annotated[Optional[str], pydantic.Field(alias="mastercardMid")] = (
        None
    )

    discover_mid: Annotated[Optional[str], pydantic.Field(alias="discoverMid")] = None

    amex_mid: Annotated[Optional[str], pydantic.Field(alias="amexMid")] = None

    updated_on: Annotated[Optional[datetime], pydantic.Field(alias="updatedOn")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["visaMid", "mastercardMid", "discoverMid", "amexMid", "updatedOn"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    WebhookDataNetworkIDUpdated.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from datetime import datetime
import httpx
from moovio_sdk.models.components import (
    amount as components_amount,
    cancellation as components_cancellation,
    cardacquiringdispute as components_cardacquiringdispute,
    cardacquiringrefund as components_cardacquiringrefund,
    facilitatorfee as components_facilitatorfee,
    moovfee as components_moovfee,
    moovfeedetails as components_moovfeedetails,
    transferdestination as components_transferdestination,
    transferfailurereason as components_transferfailurereason,
    transferlineitems as components_transferlineitems,
    transfersource as components_transfersource,
    transferstatus as components_transferstatus,
)
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated


class TransferData(BaseModel):
    transfer_id: Annotated[str, pydantic.Field(alias="transferID")]
    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]
    source: components_transfersource.TransferSource
    destination: components_transferdestination.TransferDestination
    status: components_transferstatus.TransferStatus
    r"""Status of a transfer."""
    amount: components_amount.Amount
    completed_on: Annotated[Optional[datetime], pydantic.Field(alias="completedOn")] = (
        None
    )
    failure_reason: Annotated[
        Optional[components_transferfailurereason.TransferFailureReason],
        pydantic.Field(alias="failureReason"),
    ] = None
    r"""Reason for a transfer's failure."""
    description: Optional[str] = None
    r"""An optional description of the transfer that is used on receipts and for your own internal use."""
    metadata: Optional[Dict[str, str]] = None
    r"""Free-form key-value pair list. Useful for storing information that is not captured elsewhere."""
    facilitator_fee: Annotated[
        Optional[components_facilitatorfee.FacilitatorFee],
        pydantic.Field(alias="facilitatorFee"),
    ] = None
    r"""Total or markup fee."""
    moov_fee: Annotated[Optional[int], pydantic.Field(alias="moovFee")] = None
    r"""Fees charged to your platform account for transfers."""
    moov_fee_decimal: Annotated[
        Optional[str], pydantic.Field(alias="moovFeeDecimal")
    ] = None
    r"""Same as `moovFee`, but a decimal-formatted numerical string that represents up to 9 decimal place precision."""
    moov_fee_details: Annotated[
        Optional[components_moovfeedetails.MoovFeeDetails],
        pydantic.Field(alias="moovFeeDetails"),
    ] = None
    r"""Processing and pass-through costs that add up to the moovFee."""
    moov_fees: Annotated[
        Optional[List[components_moovfee.MoovFee]], pydantic.Field(alias="moovFees")
    ] = None
    r"""Fees charged to accounts involved in the transfer."""
    group_id: Annotated[Optional[str], pydantic.Field(alias="groupID")] = None
    cancellations: Optional[List[components_cancellation.Cancellation]] = None
    refunded_amount: Annotated[
        Optional[components_amount.Amount], pydantic.Field(alias="refundedAmount")
    ] = None
    refunds: Optional[List[components_cardacquiringrefund.CardAcquiringRefund]] = None
    disputed_amount: Annotated[
        Optional[components_amount.Amount], pydantic.Field(alias="disputedAmount")
    ] = None
    disputes: Optional[List[components_cardacquiringdispute.CardAcquiringDispute]] = (
        None
    )
    sweep_id: Annotated[Optional[str], pydantic.Field(alias="sweepID")] = None
    r"""ID of the sweep that created this transfer."""
    schedule_id: Annotated[Optional[str], pydantic.Field(alias="scheduleID")] = None
    occurrence_id: Annotated[Optional[str], pydantic.Field(alias="occurrenceID")] = None
    payment_link_code: Annotated[
        Optional[str], pydantic.Field(alias="paymentLinkCode")
    ] = None
    sales_tax_amount: Annotated[
        Optional[components_amount.Amount], pydantic.Field(alias="salesTaxAmount")
    ] = None
    r"""Optional sales tax amount. `transfer.amount.value` should be inclusive of any sales tax and represents the total amount charged."""
    foreign_id: Annotated[Optional[str], pydantic.Field(alias="foreignID")] = None
    r"""Optional alias from a foreign/external system which can be used to reference this resource."""
    line_items: Annotated[
        Optional[components_transferlineitems.TransferLineItems],
        pydantic.Field(alias="lineItems"),
    ] = None
    r"""An optional collection of line items for a transfer.
    When line items are provided, their total plus sales tax must equal the transfer amount.
    """
    invoice_id: Annotated[Optional[str], pydantic.Field(alias="invoiceID")] = None
    r"""ID of the invoice that the transfer is associated with."""


@dataclass(unsafe_hash=True)
class Transfer(MoovError):
    r"""Details of a Transfer."""

    data: TransferData = field(hash=False)

    def __init__(
        self,
        data: TransferData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class DisputeTransferDetailsTypedDict(TypedDict):
    transfer_id: str
    r"""The disputed transfer's ID."""


class DisputeTransferDetails(BaseModel):
    transfer_id: Annotated[str, pydantic.Field(alias="transferID")]
    r"""The disputed transfer's ID."""


try:
    DisputeTransferDetails.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createtransferlineitem import (
    CreateTransferLineItem,
    CreateTransferLineItemTypedDict,
)
from moovio_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class CreateTransferLineItemsTypedDict(TypedDict):
    r"""An optional collection of line items for a transfer.
    When line items are provided, their total plus sales tax must equal the transfer amount.
    """

    items: List[CreateTransferLineItemTypedDict]
    r"""The list of line items."""


class CreateTransferLineItems(BaseModel):
    r"""An optional collection of line items for a transfer.
    When line items are provided, their total plus sales tax must equal the transfer amount.
    """

    items: List[CreateTransferLineItem]
    r"""The list of line items."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class PendingLitigation(str, Enum):
    BANKRUPTCY_OR_INSOLVENCY = "bankruptcy-or-insolvency"
    CONSUMER_PROTECTION_OR_CLASS_ACTION = "consumer-protection-or-class-action"
    DATA_BREACH_OR_PRIVACY = "data-breach-or-privacy"
    EMPLOYMENT_OR_WORKPLACE_DISPUTES = "employment-or-workplace-disputes"
    FRAUD_OR_FINANCIAL_CRIME = "fraud-or-financial-crime"
    GOVERNMENT_ENFORCEMENT_OR_INVESTIGATION = "government-enforcement-or-investigation"
    INTELLECTUAL_PROPERTY = "intellectual-property"
    NONE = "none"
    OTHER = "other"
    PERSONAL_INJURY_OR_MEDICAL = "personal-injury-or-medical"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cancellation import Cancellation, CancellationTypedDict
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class ReversedWithCancellationTypedDict(TypedDict):
    cancellation: CancellationTypedDict


class ReversedWithCancellation(BaseModel):
    cancellation: Cancellation



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .rtpservices import RTPServices, RTPServicesTypedDict
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class RTPInstitutionTypedDict(TypedDict):
    name: str
    r"""Name of the financial institution."""
    routing_number: str
    services: RTPServicesTypedDict


class RTPInstitution(BaseModel):
    name: str
    r"""Name of the financial institution."""

    routing_number: Annotated[str, pydantic.Field(alias="routingNumber")]

    services: RTPServices


try:
    RTPInstitution.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .walletavailablebalance import (
    WalletAvailableBalance,
    WalletAvailableBalanceTypedDict,
)
from .walletstatus import WalletStatus
from .wallettype import WalletType
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WalletTypedDict(TypedDict):
    r"""A Moov wallet to store funds for transfers."""

    wallet_id: str
    available_balance: WalletAvailableBalanceTypedDict
    partner_account_id: str
    name: str
    r"""Name of the wallet"""
    status: WalletStatus
    r"""Status of a wallet.
    - `active`: The wallet is available for use and has an enabled payment method.
    - `closed`: The wallet is no longer active and the corresponding payment method has been disabled.
    """
    wallet_type: WalletType
    r"""Type of a wallet.
    - `default`: The system-generated wallet automatically created when an account is granted the wallet capability.
    - `general`: An additional, user-defined wallet created via API or Dashboard.
    """
    description: str
    r"""Description of the wallet"""
    created_on: datetime
    metadata: NotRequired[Dict[str, str]]
    r"""Free-form key-value pair list. Useful for storing information that is not captured elsewhere."""
    closed_on: NotRequired[datetime]


class Wallet(BaseModel):
    r"""A Moov wallet to store funds for transfers."""

    wallet_id: Annotated[str, pydantic.Field(alias="walletID")]

    available_balance: Annotated[
        WalletAvailableBalance, pydantic.Field(alias="availableBalance")
    ]

    partner_account_id: Annotated[str, pydantic.Field(alias="partnerAccountID")]

    name: str
    r"""Name of the wallet"""

    status: WalletStatus
    r"""Status of a wallet.
    - `active`: The wallet is available for use and has an enabled payment method.
    - `closed`: The wallet is no longer active and the corresponding payment method has been disabled.
    """

    wallet_type: Annotated[WalletType, pydantic.Field(alias="walletType")]
    r"""Type of a wallet.
    - `default`: The system-generated wallet automatically created when an account is granted the wallet capability.
    - `general`: An additional, user-defined wallet created via API or Dashboard.
    """

    description: str
    r"""Description of the wallet"""

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]

    metadata: Optional[Dict[str, str]] = None
    r"""Free-form key-value pair list. Useful for storing information that is not captured elsewhere."""

    closed_on: Annotated[Optional[datetime], pydantic.Field(alias="closedOn")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["metadata", "closedOn"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Wallet.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DomesticPullFromCard(str, Enum):
    r"""Indicates if the card supports domestic pull-from-card transfer."""

    NOT_SUPPORTED = "not-supported"
    SUPPORTED = "supported"
    UNKNOWN = "unknown"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cardbrand import CardBrand
from .cardexpiration import CardExpiration, CardExpirationTypedDict
from .cardtype import CardType
from .transferentrymode import TransferEntryMode
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TransferTerminalCardTypedDict(TypedDict):
    r"""Describes payment card details captured with tap or in-person payment."""

    entry_mode: NotRequired[TransferEntryMode]
    r"""How the card information was entered into the point of sale terminal."""
    brand: NotRequired[CardBrand]
    r"""The card brand."""
    bin: NotRequired[str]
    card_type: NotRequired[CardType]
    r"""The type of the card."""
    expiration: NotRequired[CardExpirationTypedDict]
    r"""The expiration date of the card or token."""
    fingerprint: NotRequired[str]
    r"""Uniquely identifies a linked payment card or token.
    For Apple Pay, the fingerprint is based on the tokenized card number and may vary based on the user's device.
    This field can be used to identify specific payment methods across multiple accounts on your platform.
    """
    holder_name: NotRequired[str]
    r"""The name of the cardholder as it appears on the card."""
    issuer: NotRequired[str]
    r"""Financial institution that issued the card."""
    issuer_country: NotRequired[str]
    r"""Country where the card was issued."""
    last_four_card_number: NotRequired[str]
    r"""Last four digits of the card number"""
    application_id: NotRequired[str]
    r"""Identifier for the point of sale terminal application."""
    application_name: NotRequired[str]
    r"""Name label for the point of sale terminal application."""


class TransferTerminalCard(BaseModel):
    r"""Describes payment card details captured with tap or in-person payment."""

    entry_mode: Annotated[
        Optional[TransferEntryMode], pydantic.Field(alias="entryMode")
    ] = None
    r"""How the card information was entered into the point of sale terminal."""

    brand: Optional[CardBrand] = None
    r"""The card brand."""

    bin: Optional[str] = None

    card_type: Annotated[Optional[CardType], pydantic.Field(alias="cardType")] = None
    r"""The type of the card."""

    expiration: Optional[CardExpiration] = None
    r"""The expiration date of the card or token."""

    fingerprint: Optional[str] = None
    r"""Uniquely identifies a linked payment card or token.
    For Apple Pay, the fingerprint is based on the tokenized card number and may vary based on the user's device.
    This field can be used to identify specific payment methods across multiple accounts on your platform.
    """

    holder_name: Annotated[Optional[str], pydantic.Field(alias="holderName")] = None
    r"""The name of the cardholder as it appears on the card."""

    issuer: Optional[str] = None
    r"""Financial institution that issued the card."""

    issuer_country: Annotated[Optional[str], pydantic.Field(alias="issuerCountry")] = (
        None
    )
    r"""Country where the card was issued."""

    last_four_card_number: Annotated[
        Optional[str], pydantic.Field(alias="lastFourCardNumber")
    ] = None
    r"""Last four digits of the card number"""

    application_id: Annotated[Optional[str], pydantic.Field(alias="applicationID")] = (
        None
    )
    r"""Identifier for the point of sale terminal application."""

    application_name: Annotated[
        Optional[str], pydantic.Field(alias="applicationName")
    ] = None
    r"""Name label for the point of sale terminal application."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "entryMode",
                "brand",
                "bin",
                "cardType",
                "expiration",
                "fingerprint",
                "holderName",
                "issuer",
                "issuerCountry",
                "lastFourCardNumber",
                "applicationID",
                "applicationName",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    TransferTerminalCard.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class AmountTypedDict(TypedDict):
    currency: str
    r"""A 3-letter ISO 4217 currency code."""
    value: int
    r"""Quantity in the smallest unit of the specified currency.

    In USD this is cents, for example, $12.04 is 1204 and $0.99 is 99.
    """


class Amount(BaseModel):
    currency: str
    r"""A 3-letter ISO 4217 currency code."""

    value: int
    r"""Quantity in the smallest unit of the specified currency.

    In USD this is cents, for example, $12.04 is 1204 and $0.99 is 99.
    """



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingcountandamount import BillingCountAndAmount, BillingCountAndAmountTypedDict
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class ACHFeesTypedDict(TypedDict):
    r"""A detailed breakdown of ACH fees."""

    standard_credit: BillingCountAndAmountTypedDict
    r"""Fees for standard credit transfers."""
    same_day_credit: BillingCountAndAmountTypedDict
    r"""Fees for same-day credit transfers."""
    debits: BillingCountAndAmountTypedDict
    r"""Fees for debit transfers."""
    return_: BillingCountAndAmountTypedDict
    r"""Fees for authorized returns."""
    unauthorized_return: BillingCountAndAmountTypedDict
    r"""Fees for unauthorized returns."""
    notice_of_change: BillingCountAndAmountTypedDict
    r"""Fees for notices of change."""
    bank_account_verification: BillingCountAndAmountTypedDict
    r"""Fees for successful bank account verifications via Plaid or MX."""
    total: BillingCountAndAmountTypedDict
    r"""Total ACH fees."""


class ACHFees(BaseModel):
    r"""A detailed breakdown of ACH fees."""

    standard_credit: Annotated[
        BillingCountAndAmount, pydantic.Field(alias="standardCredit")
    ]
    r"""Fees for standard credit transfers."""

    same_day_credit: Annotated[
        BillingCountAndAmount, pydantic.Field(alias="sameDayCredit")
    ]
    r"""Fees for same-day credit transfers."""

    debits: BillingCountAndAmount
    r"""Fees for debit transfers."""

    return_: Annotated[BillingCountAndAmount, pydantic.Field(alias="return")]
    r"""Fees for authorized returns."""

    unauthorized_return: Annotated[
        BillingCountAndAmount, pydantic.Field(alias="unauthorizedReturn")
    ]
    r"""Fees for unauthorized returns."""

    notice_of_change: Annotated[
        BillingCountAndAmount, pydantic.Field(alias="noticeOfChange")
    ]
    r"""Fees for notices of change."""

    bank_account_verification: Annotated[
        BillingCountAndAmount, pydantic.Field(alias="bankAccountVerification")
    ]
    r"""Fees for successful bank account verifications via Plaid or MX."""

    total: BillingCountAndAmount
    r"""Total ACH fees."""


try:
    ACHFees.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .issuingmerchantdata import IssuingMerchantData, IssuingMerchantDataTypedDict
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class IssuedCardTransactionTypedDict(TypedDict):
    card_transaction_id: str
    issued_card_id: str
    funding_wallet_id: str
    amount: str
    r"""A decimal-formatted numerical string that represents up to 2 decimal place precision. In USD for example, 12.34 is $12.34 and 0.99 is $0.99."""
    authorized_on: datetime
    merchant_data: IssuingMerchantDataTypedDict
    created_on: datetime
    authorization_id: NotRequired[str]


class IssuedCardTransaction(BaseModel):
    card_transaction_id: Annotated[str, pydantic.Field(alias="cardTransactionID")]

    issued_card_id: Annotated[str, pydantic.Field(alias="issuedCardID")]

    funding_wallet_id: Annotated[str, pydantic.Field(alias="fundingWalletID")]

    amount: str
    r"""A decimal-formatted numerical string that represents up to 2 decimal place precision. In USD for example, 12.34 is $12.34 and 0.99 is $0.99."""

    authorized_on: Annotated[datetime, pydantic.Field(alias="authorizedOn")]

    merchant_data: Annotated[IssuingMerchantData, pydantic.Field(alias="merchantData")]

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]

    authorization_id: Annotated[
        Optional[str], pydantic.Field(alias="authorizationID")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["authorizationID"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    IssuedCardTransaction.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from moovio_sdk import utils
from moovio_sdk._hooks import HookContext
from moovio_sdk.models import components, errors, operations
from moovio_sdk.types import OptionalNullable, UNSET
from moovio_sdk.utils import get_security_from_env
from moovio_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional, Union


class Branding(BaseSDK):
    def create(
        self,
        *,
        account_id: str,
        colors: Union[components.BrandColors, components.BrandColorsTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateBrandResponse:
        r"""Create brand properties for the specified account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/branding.write` scope.

        :param account_id:
        :param colors: Brand colors for light and dark modes.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreateBrandRequest(
            account_id=account_id,
            brand_properties=components.BrandProperties(
                colors=utils.get_pydantic_model(colors, components.BrandColors),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/accounts/{accountID}/branding",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.CreateBrandGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.brand_properties,
                False,
                False,
                "json",
                components.BrandProperties,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createBrand",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateBrandResponse(
                result=unmarshal_json_response(components.BrandProperties, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BrandValidationErrorData, http_res
            )
            raise errors.BrandValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        account_id: str,
        colors: Union[components.BrandColors, components.BrandColorsTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateBrandResponse:
        r"""Create brand properties for the specified account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/branding.write` scope.

        :param account_id:
        :param colors: Brand colors for light and dark modes.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreateBrandRequest(
            account_id=account_id,
            brand_properties=components.BrandProperties(
                colors=utils.get_pydantic_model(colors, components.BrandColors),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/accounts/{accountID}/branding",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.CreateBrandGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.brand_properties,
                False,
                False,
                "json",
                components.BrandProperties,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createBrand",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateBrandResponse(
                result=unmarshal_json_response(components.BrandProperties, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BrandValidationErrorData, http_res
            )
            raise errors.BrandValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def upsert(
        self,
        *,
        account_id: str,
        colors: Union[components.BrandColors, components.BrandColorsTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpsertBrandResponse:
        r"""Create or replace brand properties for the specified account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/branding.write` scope.

        :param account_id:
        :param colors: Brand colors for light and dark modes.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpsertBrandRequest(
            account_id=account_id,
            brand_properties=components.BrandProperties(
                colors=utils.get_pydantic_model(colors, components.BrandColors),
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/accounts/{accountID}/branding",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.UpsertBrandGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.brand_properties,
                False,
                False,
                "json",
                components.BrandProperties,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="upsertBrand",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.UpsertBrandResponse(
                result=unmarshal_json_response(components.BrandProperties, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BrandValidationErrorData, http_res
            )
            raise errors.BrandValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def upsert_async(
        self,
        *,
        account_id: str,
        colors: Union[components.BrandColors, components.BrandColorsTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpsertBrandResponse:
        r"""Create or replace brand properties for the specified account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/branding.write` scope.

        :param account_id:
        :param colors: Brand colors for light and dark modes.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpsertBrandRequest(
            account_id=account_id,
            brand_properties=components.BrandProperties(
                colors=utils.get_pydantic_model(colors, components.BrandColors),
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/accounts/{accountID}/branding",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.UpsertBrandGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.brand_properties,
                False,
                False,
                "json",
                components.BrandProperties,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="upsertBrand",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.UpsertBrandResponse(
                result=unmarshal_json_response(components.BrandProperties, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.BrandValidationErrorData, http_res
            )
            raise errors.BrandValidationError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        account_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetBrandResponse:
        r"""Get brand properties for the specified account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/branding.read` scope.

        :param account_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetBrandRequest(
            account_id=account_id,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/branding",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetBrandGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getBrand",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetBrandResponse(
                result=unmarshal_json_response(components.BrandProperties, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        account_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetBrandResponse:
        r"""Get brand properties for the specified account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/branding.read` scope.

        :param account_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetBrandRequest(
            account_id=account_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/branding",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetBrandGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getBrand",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetBrandResponse(
                result=unmarshal_json_response(components.BrandProperties, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)
